(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('ws'), require('crypto')) :
	typeof define === 'function' && define.amd ? define(['exports', 'ws', 'crypto'], factory) :
	(factory((global.koaWsSocket = {}),global.WebSocket,global.crypto));
}(this, (function (exports,WebSocket,crypto) { 'use strict';

crypto = crypto && crypto.hasOwnProperty('default') ? crypto['default'] : crypto;

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var base64id = createCommonjsModule(function (module, exports) {
/*!
 * base64id v0.1.0
 */

/**
 * Module dependencies
 */



/**
 * Constructor
 */

var Base64Id = function() { };

/**
 * Get random bytes
 *
 * Uses a buffer if available, falls back to crypto.randomBytes
 */

Base64Id.prototype.getRandomBytes = function(bytes) {

  var BUFFER_SIZE = 4096;
  var self = this;  
  
  bytes = bytes || 12;

  if (bytes > BUFFER_SIZE) {
    return crypto.randomBytes(bytes);
  }
  
  var bytesInBuffer = parseInt(BUFFER_SIZE/bytes);
  var threshold = parseInt(bytesInBuffer*0.85);

  if (!threshold) {
    return crypto.randomBytes(bytes);
  }

  if (this.bytesBufferIndex == null) {
     this.bytesBufferIndex = -1;
  }

  if (this.bytesBufferIndex == bytesInBuffer) {
    this.bytesBuffer = null;
    this.bytesBufferIndex = -1;
  }

  // No buffered bytes available or index above threshold
  if (this.bytesBufferIndex == -1 || this.bytesBufferIndex > threshold) {
     
    if (!this.isGeneratingBytes) {
      this.isGeneratingBytes = true;
      crypto.randomBytes(BUFFER_SIZE, function(err, bytes) {
        self.bytesBuffer = bytes;
        self.bytesBufferIndex = 0;
        self.isGeneratingBytes = false;
      }); 
    }
    
    // Fall back to sync call when no buffered bytes are available
    if (this.bytesBufferIndex == -1) {
      return crypto.randomBytes(bytes);
    }
  }
  
  var result = this.bytesBuffer.slice(bytes*this.bytesBufferIndex, bytes*(this.bytesBufferIndex+1)); 
  this.bytesBufferIndex++; 
  
  return result;
};

/**
 * Generates a base64 id
 *
 * (Original version from socket.io <http://socket.io>)
 */

Base64Id.prototype.generateId = function () {
  var rand = new Buffer(15); // multiple of 3 for base64
  if (!rand.writeInt32BE) {
    return Math.abs(Math.random() * Math.random() * Date.now() | 0).toString()
      + Math.abs(Math.random() * Math.random() * Date.now() | 0).toString();
  }
  this.sequenceNumber = (this.sequenceNumber + 1) | 0;
  rand.writeInt32BE(this.sequenceNumber, 11);
  if (crypto.randomBytes) {
    this.getRandomBytes(12).copy(rand);
  } else {
    // not secure for node 0.4
    [0, 4, 8].forEach(function(i) {
      rand.writeInt32BE(Math.random() * Math.pow(2, 32) | 0, i);
    });
  }
  return rand.toString('base64').replace(/\//g, '_').replace(/\+/g, '-');
};

/**
 * Export
 */

exports = module.exports = new Base64Id();
});

var Socket = /** @class */ (function () {
    function Socket(ws, request, listeners, middleware) {
        this.ws = ws;
        this.request = request;
        this.middleware = middleware;
        this.update(listeners, middleware);
    }
    Socket.prototype.update = function (listeners, middleware) {
        var _this = this;
        this.ws.removeAllListeners();
        this.middleware = middleware;
        listeners.forEach(function (handlers, event) {
            if (event === 'connection') {
                return;
            }
            handlers.forEach(function (handler) {
                _this.on(event, handler);
            });
        });
    };
    /**
     * Adds a specific event and callback to this socket
     */
    Socket.prototype.on = function (event, handler) {
        var _this = this;
        this.ws.on(event, function (message) {
            var ctx = {
                event: event,
                data: message,
                request: _this.request,
                socket: _this.ws,
            };
            if (!_this.middleware) {
                handler(ctx, message);
                return;
            }
            _this.middleware(ctx).then(function () {
                handler(ctx, message);
            });
        });
    };
    return Socket;
}());

var compose = require('koa-compose');
var WsSocket = /** @class */ (function () {
    function WsSocket() {
        var _this = this;
        this.middleware = [];
        this.composed = null;
        this.connections = new Map();
        this.listeners = new Map();
        this.wss = null;
        this.onConnection = function (ws, request) {
            var socketId = base64id.generateId();
            var socketInstance = new Socket(ws, request, _this.listeners, _this.composed);
            _this.connections.set(socketId, socketInstance);
            ws.on('disconnect', function () {
                _this.connections.delete(socketId);
            });
            var handlers = _this.listeners.get('connection');
            if (handlers) {
                handlers.forEach(function (handler) {
                    handler({
                        event: 'connection',
                        data: socketInstance,
                        request: request,
                        socket: ws,
                    }, null);
                });
            }
        };
    }
    WsSocket.prototype.attach = function (app, https, tlsOpts) {
        if (https === void 0) { https = false; }
        if (tlsOpts === void 0) { tlsOpts = {}; }
        var enhancedApp = app;
        if (enhancedApp.server &&
            enhancedApp.server.constructor.name !== 'Server') {
            throw new Error("app.server already exists but it's not an http server");
        }
        if (!enhancedApp.server) {
            // Create a server if it doesn't already exists
            enhancedApp.server = https
                ? require('https').createServer(tlsOpts, enhancedApp.callback())
                : require('http').createServer(enhancedApp.callback());
            // Patch `app.listen()` to call `app.server.listen()`
            enhancedApp.listen = function listen() {
                enhancedApp.server.listen.apply(enhancedApp.server, arguments);
                return enhancedApp.server;
            };
        }
        this.wss = new WebSocket.Server({ server: enhancedApp.server });
        enhancedApp.ws = this;
        this.wss.on('connection', this.onConnection);
    };
    /**
     * Pushes a middleware on to the stack
     * @param fn <Function> the middleware function to execute
     */
    WsSocket.prototype.use = function (fn) {
        this.middleware.push(fn);
        this.composed = compose(this.middleware);
        this.updateConnections();
        return this;
    };
    /**
     * Adds a new listener to the stack
     * @param event <String> the event id
     * @param handler <Function> the callback to execute
     * @return this
     */
    WsSocket.prototype.on = function (event, handler) {
        var listeners = this.listeners.get(event) || [];
        listeners.push(handler);
        this.listeners.set(event, listeners);
        this.updateConnections();
        return this;
    };
    /**
     * Removes a listener from the event
     * @param event <String> if omitted will remove all listeners
     * @param handler <Function> if omitted will remove all from the event
     * @return this
     */
    WsSocket.prototype.off = function (event, handler) {
        if (!event) {
            this.listeners = new Map();
            this.updateConnections();
            return this;
        }
        if (!handler) {
            this.listeners.delete(event);
            this.updateConnections();
            return this;
        }
        var listeners = this.listeners.get(event) || [];
        var i = listeners.length - 1;
        while (i) {
            if (listeners[i] === handler) {
                break;
            }
            i--;
        }
        listeners.splice(i, 1);
        this.updateConnections();
        return this;
    };
    /**
     * Updates all existing connections with current listeners and middleware
     * @private
     */
    WsSocket.prototype.updateConnections = function () {
        var _this = this;
        this.connections.forEach(function (connection) {
            return connection.update(_this.listeners, _this.composed);
        });
    };
    return WsSocket;
}());

exports.WsSocket = WsSocket;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=koa-ws-socket.umd.js.map
